@@ -5,10 +5,11 @@ import re
 from datetime import datetime
 from json import JSONDecodeError
 from typing import Any, Dict, Optional
+from urllib.parse import parse_qs, urlparse
 
 from . import utils
 from .constants import FB_BASE_URL, FB_MOBILE_BASE_URL
-from .fb_types import Options, Post, RawPost, RequestFunction
+from .fb_types import Options, Post, RawPost, RequestFunction, Response, URL
 
 
 try:
@@ -35,9 +36,9 @@ def extract_group_post(raw_post: RawPost, options: Options, request_fn: RequestF
 class PostExtractor:
     """Class for Extracting fields from a FacebookPost"""
 
-    likes_regex = re.compile(r'like_def[^>]*>([0-9,.]+)')
-    comments_regex = re.compile(r'cmt_def[^>]*>([0-9,.]+)')
-    shares_regex = re.compile(r'([0-9,.]+)\s+Shares', re.IGNORECASE)
+    likes_regex = re.compile(r'([\d.KM]+)\s+(Like|left reaction)', re.IGNORECASE)
+    comments_regex = re.compile(r'([\d,.KM]+)\s+comment', re.IGNORECASE)
+    shares_regex = re.compile(r'([\d,.KM]+)\s+Share', re.IGNORECASE)
     live_regex = re.compile(r'.+(is live).+')
     link_regex = re.compile(r"href=\"https:\/\/lm\.facebook\.com\/l\.php\?u=(.+?)\&amp;h=")
 
@@ -80,6 +81,7 @@ class PostExtractor:
             'image': None,
             'image_lowquality': None,
             'images': None,
+            'images_lowquality': None,
             'video': None,
             'video_thumbnail': None,
             'video_id': None,
@@ -90,6 +92,7 @@ class PostExtractor:
             'link': None,
             'user_id': None,
             'username': None,
+            'user_url': None,
             'source': None,
             'is_live': False,
             'factcheck': None,
@@ -166,6 +169,9 @@ class PostExtractor:
             try:
                 comments = self.extract_comments_full()
                 post.update(comments)
+                if post.get("comments_full") and not post.get("comments"):
+                    post["comments"] = len(post.get("comments_full"))
+
             except Exception as ex:
                 log_warning("Exception while extracting comments: %r", ex)
 
@@ -175,8 +181,12 @@ class PostExtractor:
         return {'post_id': self.data_ft.get('top_level_post_id')}
 
     def extract_username(self) -> PartialPost:
-        username = self.element.find('h3 strong a')
-        return {'username': username[0].text} if len(username) > 0 else None
+        elem = self.element.find('h3 strong a', first=True)
+        if elem:
+            url = elem.attrs.get("href")
+            if url:
+                url = utils.urljoin(FB_BASE_URL, url)
+            return {'username': elem.text, 'user_url': url}
 
     # TODO: this method needs test for the 'has more' case and shared content
     def extract_text(self) -> PartialPost:
@@ -239,6 +249,7 @@ class PostExtractor:
         for page in page_insights.values():
             try:
                 timestamp = page['post_context']['publish_time']
+                logger.debug(f"Got exact timestamp from publish_time: {datetime.fromtimestamp(timestamp)}")
                 return {
                     'time': datetime.fromtimestamp(timestamp),
                 }
@@ -266,26 +277,19 @@ class PostExtractor:
         return {'user_id': self.data_ft['content_owner_id_new']}
 
     def extract_image_lq(self) -> PartialPost:
-        story_container = self.element.find('div.story_body_container', first=True)
-        if story_container is None:
-            return None
-        other_containers = story_container.xpath('div/div')
+        elems = self.element.find('div.story_body_container>div .img:not(.profpic)')
+        images = []
+        for elem in elems:
+            if elem.attrs.get('src'):
+                images.append(elem.attrs.get('src'))
+            elif elem.attrs.get('style'):
+                match = self.image_regex_lq.search(elem.attrs.get('style'))
+                if match:
+                    src = utils.decode_css_url(match.groups()[0])
+                    images.append(src)
 
-        for container in other_containers:
-            image_container = container.find('.img', first=True)
-            if image_container is None:
-                continue
-
-            src = image_container.attrs.get('src')
-            if src:
-                return {'image_lowquality': src}
-            style = image_container.attrs.get('style', '')
-            match = self.image_regex_lq.search(style)
-            if match:
-                src = utils.decode_css_url(match.groups()[0])
-                return {'image_lowquality': src}
-
-        return None
+        image = images[0] if images else None
+        return {"image_lowquality": image, "images_lowquality": images}
 
     def extract_link(self) -> PartialPost:
         match = self.link_regex.search(self.element.html)
@@ -333,7 +337,7 @@ class PostExtractor:
     def extract_likes(self) -> PartialPost:
         return {
             'likes': utils.find_and_search(
-                self.element, 'footer', self.likes_regex, utils.parse_int
+                self.element, 'footer', self.likes_regex, utils.convert_numeric_abbr
             )
             or 0,
         }
@@ -341,7 +345,7 @@ class PostExtractor:
     def extract_comments(self) -> PartialPost:
         return {
             'comments': utils.find_and_search(
-                self.element, 'footer', self.comments_regex, utils.parse_int
+                self.element, 'footer', self.comments_regex, utils.convert_numeric_abbr
             )
             or 0,
         }
@@ -349,35 +353,59 @@ class PostExtractor:
     def extract_shares(self) -> PartialPost:
         return {
             'shares': utils.find_and_search(
-                self.element, 'footer', self.shares_regex, utils.parse_int
+                self.element, 'footer', self.shares_regex, utils.convert_numeric_abbr
             )
             or 0,
         }
 
+    def extract_photo_link_HQ(self, response: Response) -> URL:
+        match = self.image_regex.search(response.text)
+        if match:
+            url = match.groups()[0].replace("&amp;", "&")
+            if not url.startswith("http"):
+                url = utils.urljoin(FB_MOBILE_BASE_URL, url)
+            if url.startswith(utils.urljoin(FB_MOBILE_BASE_URL, "/photo/view_full_size/")):
+                # Try resolve redirect
+                logger.debug(f"Fetching {url}")
+                redirect_response = self.request(url)
+                if not redirect_response.url.startswith(utils.urljoin(FB_MOBILE_BASE_URL, "login.php")):
+                    url = redirect_response.html.find("a", first=True).attrs.get("href").replace("&amp;", "&")
+            return url
+        else:
+            return None
+
     def extract_photo_link(self) -> PartialPost:
         if not self.options.get("allow_extra_requests", True):
             return None
         images = []
-        matches = list(self.photo_link.finditer(self.element.html))
-        if not matches:
-            matches = self.photo_link_2.finditer(self.element.html)
+        photo_links = self.element.find("div.story_body_container>div a[href*='photo.php'],a[href*='/photos/']")
+        total_photos_in_gallery = len(photo_links)
+        if len(photo_links) == 4 and photo_links[-1].text:
+            total_photos_in_gallery = 4 + int(photo_links[-1].text.strip("+"))
+            logger.debug(f"{total_photos_in_gallery} total photos in gallery")
 
-        for match in matches:
-            url = utils.urljoin(FB_MOBILE_BASE_URL, match.groups()[0])
+        # This gets up to 4 images in gallery
+        for link in photo_links:
+            url = link.attrs["href"]
+            if "photoset_token" in url:
+                query = parse_qs(urlparse(url).query)
+                photo_id = query["photo"][0]
+                profile_id = query["profileid"][0]
+                url = f"/photo.php?fbid={photo_id}&profileid={profile_id}"
 
+            url = utils.urljoin(FB_MOBILE_BASE_URL, url)
+            logger.debug(f"Fetching {url}")
             response = self.request(url)
-            html = response.text
-            match = self.image_regex.search(html)
-            if match:
-                url = match.groups()[0].replace("&amp;", "&")
-                if not url.startswith("http"):
-                    url = utils.urljoin(FB_MOBILE_BASE_URL, url)
-                if url.startswith(utils.urljoin(FB_MOBILE_BASE_URL, "/photo/view_full_size/")):
-                    # Try resolve redirect
-                    response = self.request(url)
-                    if not response.url.startswith(utils.urljoin(FB_MOBILE_BASE_URL, "login.php")):
-                        url = response.html.find("a", first=True).attrs.get("href").replace("&amp;", "&")
-                images.append(url)
+            images.append(self.extract_photo_link_HQ(response))
+
+        while len(images) < total_photos_in_gallery:
+            # More photos to fetch. Follow the right arrow link of the last image we were on
+            url = response.html.find('a.touchable[data-gt=\'{"tn":"+="}\']', first=True).attrs["href"]
+            if not url.startswith("http"):
+                url = utils.urljoin(FB_MOBILE_BASE_URL, url)
+            logger.debug(f"Fetching {url}")
+            response = self.request(url)
+            images.append(self.extract_photo_link_HQ(response))
         image = images[0] if images else None
         return {"image": image, "images": images}
 
@@ -601,22 +629,21 @@ class PostExtractor:
 
     def parse_comment(self, comment):
         comment_id = comment.attrs.get("id")
+
+        profile_picture = comment.find(".profpic.img", first=True)
+        name = profile_picture.attrs.get("alt") or profile_picture.attrs.get("aria-label")
+        name = name.split(",")[0]
+
+        url = profile_picture.element.getparent().attrib.get("href")
+        if url:
+            url = utils.urljoin(FB_BASE_URL, url)
+
         first_link = comment.find("div:not([data-sigil])>a[href]:not([data-click]):not([data-store]):not([data-sigil])", first=True)
         comment_body_elem = comment.find('[data-sigil="comment-body"]', first=True)
         commenter_meta = None
         if first_link:
-            url = utils.urljoin(FB_BASE_URL, first_link.attrs.get("href"))
-            name = first_link.text
-            if "\n" in name:
-                commenter_meta, name = name.split("\n")
-        else:
-            # Adjacent div to comment body, if not logged in. No link to user page available in that case
-            url = None
-            elem = comment_body_elem.element.getprevious()
-            if elem is not None:
-                name = elem.text
-            else:
-                name = None
+            if "\n" in first_link.text:
+                commenter_meta = first_link.text.split("\n")[0]
 
         text = comment_body_elem.text
         # Try to extract from the abbr element
@@ -642,16 +669,27 @@ class PostExtractor:
         logger.debug(f"Fetching {url}")
         response = self.request(url)
         elem = response.html.find('div[data-sigil="m-mentions-expand"]', first=True)
+        if not elem:
+            logger.warning("No comments found on page")
+            return
         comments = list(elem.find('div[data-sigil="comment"]'))
+        limit = 5000
+        comments_opt = self.options.get('comments')
+        if type(comments_opt) in [int, float]:
+            limit = comments_opt
         more = elem.find("a", containing="View more comments", first=True)
-        while more:
+        while more and len(comments) < limit:
             url = utils.urljoin(FB_MOBILE_BASE_URL, more.attrs.get("href"))
             logger.debug(f"Fetching {url}")
             response = self.request(url)
             elem = response.html.find('div[data-sigil="m-mentions-expand"]', first=True)
+            if not elem:
+                logger.warning("No comments found on page")
+                break
             more_comments = elem.find('div[data-sigil="comment"]')
             comments.extend(more_comments)
             more = response.html.find("a", containing="View more comments", first=True)
+
         logger.debug(f"Found {len(comments)} comments")
         results = []
         for comment in comments:
